//*****************************************************************************
//
//      Class:          IconImage
//      Namespace:      Kerr.IconBrowser
//      Date created:   15 June 2001
//      Copyright:       Kenny Kerr 2002    http://www.kennyandkarin.com
//
//*****************************************************************************

#include "stdafx.h"
#include "IconImage.h"
#include "Library.h"

namespace Kerr { namespace IconBrowser
{

    //
    // Creates a new IconImage object to represent a RT_ICON resource.
    // library: The library that contains the resource.
    // pResourceName: The name of the resource.
    //
    IconImage::IconImage(Library* library,
                         System::IntPtr pResourceName)
    {
        Debug::Assert(0 != library);
        Debug::Assert(System::IntPtr::Zero != pResourceName);

        // Find the resource with the given name.

        System::IntPtr hIconInfo = library->FindResource(pResourceName,
                                                         WindowsAPI::RT_ICON);

        // Load the resource and get the resource bits.

        System::IntPtr hIconRes = library->LoadResource(hIconInfo);
        System::IntPtr pDibBits = WindowsAPI::LockResource(hIconRes);

        // We make a local copy of the DIB bits as pDibBits may be 
        // freed when the module is unloaded.
        m_arDibBits = new System::Byte[library->SizeofResource(hIconInfo)];

        System::Runtime::InteropServices::Marshal::Copy(pDibBits, 
                                                        m_arDibBits, 
                                                        0, 
                                                        m_arDibBits.Length);

        System::Byte __pin* pBytes = &m_arDibBits[0];
        WindowsAPI::BITMAPINFO* pBitmapInfo = reinterpret_cast<WindowsAPI::BITMAPINFO*>(pBytes);

        // The header height value is the combined height of XOR and AND masks.
        // So we simply divide by two to get the image height.
        m_iHeight = pBitmapInfo->bmiHeader.biHeight / 2;
        
        m_iWidth = pBitmapInfo->bmiHeader.biWidth;
        m_iColors = pBitmapInfo->bmiHeader.biPlanes * pBitmapInfo->bmiHeader.biBitCount;
    }

    //
    // Gets the size of the icon image.
    //
    int IconImage::get_Size()
    {
        return m_arDibBits.Length;
    }

    //
    // Writes the directory entry to the stream.
    //
    void IconImage::WriteDirectoryEntry(System::IO::BinaryWriter* writer,
                                        WindowsAPI::UINT uiImageOffset)
    {
        Debug::Assert(0 != writer);

        WindowsAPI::FILEICONDIRENTRY data = { 0 };
        data.dwImageOffset = uiImageOffset;

        data.bWidth = static_cast<WindowsAPI::BYTE>(m_iWidth);
        data.bHeight = static_cast<WindowsAPI::BYTE>(m_iHeight);

        System::Byte __pin* pBytes = &m_arDibBits[0];
        WindowsAPI::BITMAPINFO* pBitmapInfo = reinterpret_cast<WindowsAPI::BITMAPINFO*>(pBytes);

        data.wPlanes = pBitmapInfo->bmiHeader.biPlanes;
        data.wBitCount = pBitmapInfo->bmiHeader.biBitCount;

        WindowsAPI::BYTE bColorCount = static_cast<WindowsAPI::BYTE>(data.wPlanes * data.wBitCount);

        if (8 <= bColorCount)
        {
            data.bColorCount = 0;
        }
        else
        {
            data.bColorCount = 1 << bColorCount;
        }

        data.dwBytesInRes = get_Size();

        writer->Write(data.bWidth);
        writer->Write(data.bHeight);
        writer->Write(data.bColorCount);
        writer->Write(data.bReserved);
        writer->Write(data.wPlanes);
        writer->Write(data.wBitCount);
        writer->Write(data.dwBytesInRes);
        writer->Write(data.dwImageOffset);
    }

    //
    // Writes the image data to the stream.
    //
    void IconImage::WriteImage(System::IO::BinaryWriter* writer)
    {
        Debug::Assert(0 != writer);

        // When we write the BITMAPINFO the image size must be set to zero.
        // So we cache it here and restore it after we're done writing.

        System::Byte __pin* pBytes = &m_arDibBits[0];
        WindowsAPI::BITMAPINFO* pBitmapInfo = reinterpret_cast<WindowsAPI::BITMAPINFO*>(pBytes);

        WindowsAPI::DWORD biSizeImage = pBitmapInfo->bmiHeader.biSizeImage;
        pBitmapInfo->bmiHeader.biSizeImage = 0;

        writer->Write(m_arDibBits);

        pBitmapInfo->bmiHeader.biSizeImage = biSizeImage;
    }
}}