//*****************************************************************************
//
//      Class:          IconResource
//      Namespace:      Kerr.IconBrowser
//      Date created:   15 June 2001
//      Copyright:       Kenny Kerr 2002    http://www.kennyandkarin.com
//
//*****************************************************************************

#include "stdafx.h"
#include "IconResource.h"
#include "Library.h"
#include "IconImage.h"
#include "Using.h"

namespace Kerr { namespace IconBrowser
{
    //
    // Create an IconResource object to represent the icon resource in the
    // given library.
    //
    IconResource::IconResource(Library* library,
                               System::IntPtr pResourceName) :
        m_pName(pResourceName),
        m_strText(System::String::Empty)
    {
        Debug::Assert(0 != library);
        Debug::Assert(System::IntPtr::Zero != pResourceName);

        // Get the RT_GROUP_ICON resource data.

        System::IntPtr hGroupInfo = library->FindResource(m_pName, 
                                                          WindowsAPI::RT_GROUP_ICON);

        System::IntPtr hGroupRes = library->LoadResource(hGroupInfo);

        WindowsAPI::MEMICONDIR* pDirectory = static_cast<WindowsAPI::MEMICONDIR*>(WindowsAPI::LockResource(hGroupRes));

        // Get the RT_ICON resource data for each icon in the RT_GROUP_ICON.

        m_images = new IconImage*[pDirectory->wCount];

        for (WindowsAPI::USHORT i = 0; i < pDirectory->wCount; ++i)
        {
            WindowsAPI::MEMICONDIRENTRY& entry = pDirectory->arEntries[i];  //GetDirectoryEntry(pDirectory, i);

            m_images[i] = new IconImage(library, 
                                        entry.wId);
        }
    }

    //
    // Gets the name of the resource as a string.
    //
    System::String* IconResource::get_Text()
    {
        if (0 == m_strText->Length)
        {
            if (WindowsAPI::IS_INTRESOURCE(m_pName))
            {
                m_strText = m_pName.ToString();
            }
            else
            {
                m_strText = System::Runtime::InteropServices::Marshal::PtrToStringAuto(m_pName);
            }
        }

        return m_strText;
    }

    //
    // Saves the icon resource to a .ICO file.
    //
    void IconResource::Save(System::String* strFileName)
    {
        Debug::Assert(0 != strFileName);

        System::IO::Stream* stream = new System::IO::FileStream(strFileName, 
                                                                System::IO::FileMode::Create, 
                                                                System::IO::FileAccess::Write, 
                                                                System::IO::FileShare::None);

        Using<System::IO::BinaryWriter*> writer = new System::IO::BinaryWriter(stream);

        WriteHeader(writer);
        WriteDirectoryEntries(writer);
        WriteImages(writer);
    }

    //
    // Writes the .ICO file header data to the stream.
    //
    void IconResource::WriteHeader(System::IO::BinaryWriter* writer)
    {
        Debug::Assert(0 != writer);

        // Write the reserved WORD (0).
        WindowsAPI::WORD word = 0;
        writer->Write(word);

        // Write the resource type (1 for icons).
        word = 1;
        writer->Write(word);

        // Write the image count.
        word = (WindowsAPI::WORD) m_images->get_Length();
        writer->Write(word);
    }
    
    //
    // Writes the icon directory entries to the stream.
    //
    void IconResource::WriteDirectoryEntries(System::IO::BinaryWriter* writer)
    {
        Debug::Assert(0 != writer);

        for (int i = 0; i < m_images->get_Length(); ++i)
        {
            IconImage* image = m_images[i];
            image->WriteDirectoryEntry(writer, GetIcoImageOffset(i));
        }
    }
    
    //
    // Writes the icon image data to the stream.
    //
    void IconResource::WriteImages(System::IO::BinaryWriter* writer)
    {
        Debug::Assert(0 != writer);

        for (int i = 0; i < m_images->get_Length(); ++i)
        {
            IconImage* image = m_images[i];
            image->WriteImage(writer);
        }
    }
    
    //
    // Gets the offset of the image data from the beginning of the .ICO 
    // file. This is used by WriteDirectoryEntries to write the icon 
    // directory entries.
    //
    int IconResource::GetIcoImageOffset(int index)
    {
        // Space for the ICO header.
        int iOffset = 3 * sizeof (WindowsAPI::WORD);

        // Space for the FILEICONDIRENTRY structures.
        iOffset += m_images->get_Length() * sizeof (WindowsAPI::FILEICONDIRENTRY);

        // Space for any preceding images.
        for (int i = 0; i < index; ++i)
        {
            IconImage* image = m_images[i];
            iOffset += image->get_Size();
        }

        return iOffset;
    }
}}
