//*****************************************************************************
//
//      Class:          Library
//      Namespace:      Kerr.IconBrowser
//      Date created:   15 June 2001
//      Copyright:       Kenny Kerr 2002    http://www.kennyandkarin.com
//
//*****************************************************************************

#include "stdafx.h"
#include "Library.h"

namespace Kerr { namespace IconBrowser
{
    //
    // Maps the specified module into the address space of the calling 
    // process. The module will be freed when the Library object is 
    // disposed.
    //
    Library::Library(System::String* strFileName,
                     WindowsAPI::DWORD dwFlags) :
        m_bOwn(true)
    {
        Debug::Assert(0 != strFileName);

        m_hModule = WindowsAPI::LoadLibraryEx(strFileName,
                                              System::IntPtr::Zero,
                                              dwFlags);

        if (System::IntPtr::Zero == m_hModule)
        {
            WindowsAPI::ThrowLastError();
        }
    }

    //
    // Creates a Library object attached to the given module. If 
    // bTakeOwnership is true the module will be freed when the Library 
    // object is disposed.
    //
    Library::Library(System::IntPtr hModule,
                     bool bTakeOwnership) :
        m_hModule(hModule),
        m_bOwn(bTakeOwnership)
    {
        Debug::Assert(System::IntPtr::Zero != hModule);
    }

    //
    // Frees the underlying module.
    //
    void Library::Dispose()
    {
        if (m_bOwn && System::IntPtr::Zero != m_hModule)
        {
            WindowsAPI::FreeLibrary(m_hModule);
        }

        m_hModule = System::IntPtr::Zero;
    }

    //
    // Gets a handle to the underlying module.
    //
    System::IntPtr Library::get_Handle()
    {
        return m_hModule;
    }

    //
    // Searches the library for each resource of the specified type and 
    // passes either the name or the ID of each resource it locates to the
    // delegate.
    //
    void Library::EnumResourceNames(System::IntPtr pType,
                                    WindowsAPI::EnumResNameProc* enumFunc,
                                    System::IntPtr param)
    {
        Debug::Assert(System::IntPtr::Zero != pType);
        Debug::Assert(0 != enumFunc);

        if (!WindowsAPI::EnumResourceNames(m_hModule,
                                           pType,
                                           enumFunc,
                                           param))
        {
            // EnumResourceNames return failure if no resource of the specified
            // type is found. In this case we simply treat it as success 
            // without any results, and only throw an exception if something 
            // else went wrong.

            WindowsAPI::DWORD dwErrorCode = WindowsAPI::GetLastError();

            if (WindowsAPI::NO_ERROR != dwErrorCode && 
                WindowsAPI::ERROR_RESOURCE_TYPE_NOT_FOUND != dwErrorCode)
            {
                WindowsAPI::ThrowLastError();
            }
        }
    }

    //
    // Determines the location of a resource with the specified type and 
    // name.
    //
    System::IntPtr Library::FindResource(System::IntPtr pName,
                                         System::IntPtr pType)
    {
        Debug::Assert(System::IntPtr::Zero != pName);
        Debug::Assert(System::IntPtr::Zero != pType);

        System::IntPtr hResource = WindowsAPI::FindResource(m_hModule,
                                                            pName,
                                                            pType);

        if (System::IntPtr::Zero == hResource)
        {
            WindowsAPI::ThrowLastError();
        }

        return hResource;
    }

    //
    // Loads the specified resource into memory.
    //
    System::IntPtr Library::LoadResource(System::IntPtr hResource)
    {
        Debug::Assert(System::IntPtr::Zero != hResource);

        System::IntPtr hGlobal = WindowsAPI::LoadResource(m_hModule,
                                                          hResource);

        if (System::IntPtr::Zero == hGlobal)
        {
            WindowsAPI::ThrowLastError();
        }

        return hGlobal;
    }

    //
    // Gets the size, in bytes, of the specified resource.
    //
    WindowsAPI::DWORD Library::SizeofResource(System::IntPtr hResource)
    {
        Debug::Assert(System::IntPtr::Zero != hResource);

        WindowsAPI::DWORD dwByteCount = WindowsAPI::SizeofResource(m_hModule,
                                                                   hResource);

        if (0 == dwByteCount)
        {
            WindowsAPI::ThrowLastError();
        }

        return dwByteCount;
    }

    //
    // Loads the specified icon resource from the library.
    //
    System::Drawing::Icon* Library::LoadIcon(System::IntPtr pName)
    {
        Debug::Assert(System::IntPtr::Zero != pName);

        System::IntPtr hIcon = WindowsAPI::LoadIcon(m_hModule,
                                                    pName);

        if (System::IntPtr::Zero == hIcon)
        {
            WindowsAPI::ThrowLastError();
        }

        return System::Drawing::Icon::FromHandle(hIcon);
    }
}}
