//*****************************************************************************
//
//      Class:          MainWindow
//      Namespace:      Kerr.IconBrowser
//      Date created:   15 June 2001
//      Copyright:       Kenny Kerr 2002    http://www.kennyandkarin.com
//
//*****************************************************************************

#include "stdafx.h"
#include "MainWindow.h"
#include "Library.h"
#include "IconResource.h"
#include "IconItem.h"
#include "Using.h"

using namespace System::Windows::Forms;

namespace Kerr { namespace IconBrowser
{
    //
    // Creates a new MainWindow object for the application.
    //
    MainWindow::MainWindow()
    {
        AcceptButton = m_btnGo;
        ClientSize = System::Drawing::Size(464, 406);
        MinimumSize = System::Drawing::Size(300, 200);
        Text = S"Icon Browser 1.0 by Kenny Kerr";

        Font = new System::Drawing::Font(S"Tahoma", 
                                         8.25F, 
                                         System::Drawing::FontStyle::Regular, 
                                         System::Drawing::GraphicsUnit::Point);

        LoadWindowIcon();
        CreateBrowserView();
        CreateControlPanel();
    }

    //
    // Displays the Open File dialog box, allowing the user to choose a 
    // resource file to search for icons.
    //
    void MainWindow::BrowseEventHandler(System::Object*, System::EventArgs*)
    {
        Using<OpenFileDialog*> dlg = new OpenFileDialog();

        dlg->Filter = S"Icon Resource Files (*.exe;*.dll)|*.exe;*.dll|All Files (*.*)|*.*";
        dlg->Title = S"Choose a File";

        if (DialogResult::OK == dlg->ShowDialog(this))
        {
            m_edtFileName->Text = dlg->FileName;
            m_edtFileName->Focus();
        }
    }

    //
    // Enables the OK button if the edit control contains some text, 
    // hopefully a file name...
    //
    void MainWindow::FileNameChangedEventHandler(System::Object*, System::EventArgs*)
    {
        m_btnGo->Enabled = (0 < m_edtFileName->TextLength);
    }

    //
    // Searches the user-supplied file for icon resources and adds any 
    // that are found to the list.
    //
    void MainWindow::GoEventHandler(System::Object*, System::EventArgs*)
    {
        try
        {
            Cursor::Current = Cursors::WaitCursor;

            try
            {
                // Load the selected library and populate the list view 
                // with any icons that are found.

                Using<Library*> library = new Library(m_edtFileName->Text,
                                                      WindowsAPI::LOAD_LIBRARY_AS_DATAFILE);

                m_lvBrowser->Clear();
                m_ilIcons->Images->Clear();

                library->EnumResourceNames(WindowsAPI::RT_GROUP_ICON,
                                           new WindowsAPI::EnumResNameProc(this, AddIconToList),
                                           System::IntPtr::Zero);

                // If icons were found the select the first icon, 
                // otherwise notify the user that we couldn't find any.

                if (0 < m_lvBrowser->Items->Count)
                {
                    m_lvBrowser->Focus();
                    m_lvBrowser->Items->get_Item(0)->Selected = true;
                }
                else
                {
                    MessageBox::Show(this, 
                                     S"The specified file does not contain any icons.", 
                                     S"Icon Browser", 
                                     MessageBoxButtons::OK, 
                                     MessageBoxIcon::Information);
                }
            }
            catch (System::ComponentModel::Win32Exception* e)
            {
                // If the message contains an insert marker, its probably 
                // looking for the file name. So we resolve it and then rethrow
                // the exception. Although this isn't 100% foolproof its better
                // than letting the user see "%1" in the most common case.

                System::String* strMessage = e->Message->Replace(S"%1", 
                                                                 m_edtFileName->Text);

                throw new System::Exception(strMessage, e);
            }
        }
        catch (System::Exception* e)
        {
            System::String* strMessage = System::String::Format(S"Failed to extract any icons from the specified file.\n\n{0}",
                                                                e->Message);

            MessageBox::Show(this, 
                             strMessage, 
                             S"Icon Browser", 
                             MessageBoxButtons::OK, 
                             MessageBoxIcon::Error);
        }
        __finally
        {
            Cursor::Current = Cursors::Default;
        }

        UpdateExportButton();
    }

    //
    // Displays the Save As dialog box, allowing the user to indicate 
    // where to save the .ICO file. Then it writes the icon to the chosen
    // file.
    //
    void MainWindow::ExportEventHandler(System::Object*, System::EventArgs*)
    {
        IconItem* item = static_cast<IconItem*>(m_lvBrowser->SelectedItems->get_Item(0));

        Using<SaveFileDialog*> dlg = new SaveFileDialog();

        dlg->DefaultExt = S"ico";
        dlg->Filter = S"Icon Files (*.ico)|*.ico|All Files (*.*)|*.*";
        dlg->Title = S"Save Icon As";
        dlg->FileName = System::String::Format(S"{0}.ico", item->Text);

        if (DialogResult::OK == dlg->ShowDialog(this))
        {
            item->IconResource->Save(dlg->FileName);
        }
    }

    //
    // Enables the Export button if the user has select an icon.
    //
    void MainWindow::SelectionChangedEventHandler(System::Object*, System::EventArgs*)
    {
        UpdateExportButton();
    }

    //
    // Shows the context menu if the user clicked on an icon.
    //
    void MainWindow::ContextMenuPopupEventHandler(System::Object*, System::EventArgs*)
    {
        m_miExport->Visible = (1 == m_lvBrowser->SelectedItems->Count);
    }

    //
    // Loads the icon from this module and sets it as the form's icon.
    //
    void MainWindow::LoadWindowIcon()
    {
        System::Reflection::Assembly* assembly = System::Reflection::Assembly::GetExecutingAssembly();

        System::IntPtr hIcon = WindowsAPI::ExtractIcon(System::IntPtr::Zero,
                                                       assembly->Location,
                                                       0);

        if (System::IntPtr::Zero != hIcon)
        {
            Icon = System::Drawing::Icon::FromHandle(hIcon);
        }
    }

    //
    // Creates the panel that houses the controls for opening and saving files.
    //
    void MainWindow::CreateControlPanel()
    {
        // Create the panel.
        Panel* panel = new Panel();
        Controls->Add(panel);
        panel->Dock = DockStyle::Top;
        panel->Height = 40;
        panel->TabIndex = 0;

        // Create the "Browse..." button.
        Button* btnBrowse = new Button();
        panel->Controls->Add(btnBrowse);
        btnBrowse->FlatStyle = FlatStyle::System;
        btnBrowse->Location = System::Drawing::Point(8, 8);
        btnBrowse->Size = System::Drawing::Size(72, 24); 
        btnBrowse->TabIndex = 0;
        btnBrowse->Text = S"&Browse...";
        btnBrowse->add_Click(new System::EventHandler(this, BrowseEventHandler));

        // Create the file name text box.
        m_edtFileName = new TextBox();
        panel->Controls->Add(m_edtFileName);
        m_edtFileName->Anchor = AnchorStyles(AnchorStyles::Top | AnchorStyles::Left | AnchorStyles::Right);
        m_edtFileName->Location = System::Drawing::Point(88, 10);
        m_edtFileName->Size = System::Drawing::Size(224, 21);
        m_edtFileName->TabIndex = 1;
        m_edtFileName->add_TextChanged(new System::EventHandler(this, FileNameChangedEventHandler));

        // Create the "Go" button.
        m_btnGo = new Button();
        panel->Controls->Add(m_btnGo);
        AcceptButton = m_btnGo;
        m_btnGo->Anchor = AnchorStyles(AnchorStyles::Top | AnchorStyles::Right);
        m_btnGo->Enabled = false;
        m_btnGo->FlatStyle = FlatStyle::System;
        m_btnGo->Location = System::Drawing::Point(320, 8);
        m_btnGo->Size = System::Drawing::Size(32, 24);
        m_btnGo->TabIndex = 2;
        m_btnGo->Text = S"&Go";
        m_btnGo->add_Click(new System::EventHandler(this, GoEventHandler));

        // Create the "Export Icon..." button.
        m_btnExport = new Button();
        panel->Controls->Add(m_btnExport);
        m_btnExport->Anchor = AnchorStyles(AnchorStyles::Top | AnchorStyles::Right);
        m_btnExport->Enabled = false;
        m_btnExport->FlatStyle = FlatStyle::System;
        m_btnExport->Location = System::Drawing::Point(368, 8);
        m_btnExport->Size = System::Drawing::Size(88, 24);
        m_btnExport->TabIndex = 3;
        m_btnExport->Text = S"&Export Icon...";
        m_btnExport->add_Click(new System::EventHandler(this, ExportEventHandler));
    }

    //
    // Creates the main view that dispays the icons.
    //
    void MainWindow::CreateBrowserView()
    {
        // Create the ImageList that will contain the icon images.
        m_ilIcons = new ImageList();
        m_ilIcons->ColorDepth = ColorDepth::Depth32Bit;
        m_ilIcons->ImageSize = System::Drawing::Size(32, 32);
        m_ilIcons->TransparentColor = System::Drawing::Color::Transparent;

        // Create the context menu for the ListView.
        System::Windows::Forms::ContextMenu* contextMenu = new System::Windows::Forms::ContextMenu();
        contextMenu->add_Popup(new System::EventHandler(this, ContextMenuPopupEventHandler));

        // Add the context menu item.
        m_miExport = contextMenu->MenuItems->Add(S"&Export Icon",
                                                 new System::EventHandler(this, ExportEventHandler));

        // Create the ListView that will display the icons.
        m_lvBrowser = new ListView();
        Controls->Add(m_lvBrowser);
        m_lvBrowser->Dock = DockStyle::Fill;
        m_lvBrowser->HideSelection = false;
        m_lvBrowser->MultiSelect = false;
        m_lvBrowser->LargeImageList = m_ilIcons;
        m_lvBrowser->ContextMenu = contextMenu;
        m_lvBrowser->add_SelectedIndexChanged(new System::EventHandler(this, SelectionChangedEventHandler));
    }

    //
    // Enables the Export button if the user has select an icon.
    //
    void MainWindow::UpdateExportButton()
    {
        m_btnExport->Enabled = (1 == m_lvBrowser->SelectedItems->Count);
    }

    //
    // This is the callback that gets called for any icon resources that 
    // are found. Create an IconResource object to represent it and an 
    // item to the list.
    //
    bool MainWindow::AddIconToList(System::IntPtr hModule,
                                   System::IntPtr /*pType*/,
                                   System::IntPtr pName,
                                   System::IntPtr /*param*/)
    {
        Debug::Assert(System::IntPtr::Zero != hModule);
        Debug::Assert(System::IntPtr::Zero != pName);

        try
        {
            Library* library = new Library(hModule, false);
            IconResource* iconResource = new IconResource(library, pName);
            
            m_ilIcons->Images->Add(library->LoadIcon(pName));

            int iImageIndex = m_ilIcons->Images->Count - 1;
            m_lvBrowser->Items->Add(new IconItem(iconResource, iImageIndex));
        }
        catch (System::ComponentModel::Win32Exception* e)
        {
            // We simply suppress any failure in loading an icon resource.
            // In this way Icon Browser will still be able to display other
            // icons that may have loaded successfully.

            Debug::WriteLine(System::String::Format(S"Ignoring exception: {0}",
                                                    e->Message));
        }

        // Continue until there are no more icons.
        return true;
    }
}}
